%
% Plot results from an Aardvark Design Sweep.
% This plotting variant is supposed to be used with one or more CSV files
% all containing a single sweep point each.
%
% Author: Mikael Henriksson (2022)
%


% -----------------------  SETTINGS BEGIN  ------------------------------ %

%
% Input CSV files to include in the results
%
IN_FILES = [
    "./sweep_accumulator_fix-32bit/result.csv", ...
    "./sweep_accumulator_fix-64bit/result.csv", ...
    "./sweep_accumulator_float-32bit/result.csv", ...
    "./sweep_accumulator_float-64bit/result.csv", ...
];

%
% Legend strings to use for the data from the input files
%
LEGENDS = [
    "Fixed-point 32-bit", ...
    "Fixed-point 64-bit", ...
    "Floating-point 32-bit", ...
    "Floating-point 64-bit", ...
];

%
% Other settings
%
EXCLUDE_NEGATIVE_SLACK = false;

% ------------------------  SETTINGS END  ------------------------------- %


%
% Loop over all the in-data and produce the plot
%
figure;
hold on;
for in_idx = 1:length(IN_FILES)
    
    % Current data points setting
    filename = IN_FILES(in_idx);
    legend_name = LEGENDS(in_idx);
    current_data = readtable(IN_FILES(in_idx));
    
    % Extract table data
    period = table2array(current_data(:,1));
    size = table2array(current_data(:,2));
    power = table2array(current_data(:,3));
    slack = table2array(current_data(:,4));
    
    % Arrange data points
    critical_path = period - min(zeros(length(period), 1), slack);
    if EXCLUDE_NEGATIVE_SLACK
        fig_periods = period(slack >= 0);
        fig_size = size(slack >= 0);
    else
        [fig_periods, I] = sort(critical_path, 'descend');
        fig_size = size(I);
    end
    
    % Plot data points
    plot( ...
        1000./fig_periods, fig_size, ...
        'LineWidth', 2 ...
    );
    
end

xlabel('Frequency MHz');
ylabel('Area um^2');
grid on;

legend(LEGENDS)
