%
% Plot results from an Aardvark Design Sweep.
% This plotting variant is supposed to be used with a single CSV file,
% which can contain one or more generic sweeps.
%
% Author: Mikael Henriksson (2022)
%


% -----------------------  SETTINGS BEGIN  ------------------------------ %

%
% Input CSV file (possibly including multiple generic sweeps)
% to include in the results
%
IN_FILE = './sweep_accumulator_fix-32bit/result.csv';

%
% Legend(s) for the (possible mulptiple generic sweeps) sweeps
%
LEGENDS = [ ...
    'Processing element (32 bit)', ...
];

%
% Other settings
%
EXCLUDE_NEGATIVE_SLACK = false;

% ------------------------  SETTINGS END  ------------------------------- %

% Read data from disk
in_data = readtable(IN_FILE);

% Is this a generic sweep?
generic_sweep = width(in_data) > 5;

% Create plot based on design generics
figure;
hold on;
if generic_sweep
    % Design generics has been sweept: one generic sweep point per line in
    % the frequency/area figure.
    design_list = string(table2array(unique(in_data(:,1), 'stable')));
    designs = length(design_list);
    hold on;
    for n = 1:designs
        % Extract table data
        data = in_data(table2cell(in_data(:,1)) == design_list(n), :);
        period = table2array(data(:,2));
        size = table2array(data(:,3));
        power = table2array(data(:,4));
        slack = table2array(data(:,5));
        critical_path = period - min(zeros(length(period), 1), slack);

        % Conditionally exlude negative slack points
        if EXCLUDE_NEGATIVE_SLACK
            fig_periods = period(slack >= 0);
            fig_size = size(slack >= 0);
        else
            [fig_periods, I] = sort(critical_path, 'descend');
            fig_size = size(I);
        end
        
        % Plot data points
        plot( ...
            1000./fig_periods, fig_size, ...
            'LineWidth', 2 ...
        );
    end
    legend(LEGENDS);
else
    % No design generics has been sweept: show the sweep with a signle line
    % in the frequency/area figure.
    
    % Extract table data
    period = table2array(in_data(:,1));
    size = table2array(in_data(:,2));
    power = table2array(in_data(:,3));
    slack = table2array(in_data(:,4));
    critical_path = period - min(zeros(length(period), 1), slack);
    
    % Conditionally exlude negative slack points
    if EXCLUDE_NEGATIVE_SLACK
        fig_periods = period(slack >= 0);
        fig_size = size(slack >= 0);
    else
        [fig_periods, I] = sort(critical_path, 'descend');
        fig_size = size(I);
    end
    
    % Plot data points
    plot( ...
        1000./fig_periods, fig_size, ...
        'LineWidth', 2 ...
    );
    legend(LEGENDS);
end

xlabel('Frequency MHz');
ylabel('Area um^2');
grid on;
