#
# Place and route global setting. This file is to be used with the provided '.synopsys_dc.setup' file
# Author: Mikael Henriksson and Olle Hansson (2022)
#

set DESIGN_NAME "cd_top_reg"
set MW_LIBRARY_NAME "${DESIGN_NAME}-lib"
set VERILOG_NETLIST [list "${DESIGN_NAME}-post-synth.v"]
set SDC_CONSTRAINT [list "${DESIGN_NAME}-post-synth.sdc"] 

#
# Maximum number of allowed cores during place-and-route. Linköping University (ISY) has a license limit of 16 cores
# per place-and-route operation (as of Nov. 2022).
#
set_host_options -max_cores 16

#
# From the user manual 'Foundation_Synopsys_BE_TechnoKit_cmos028FDSOI_6U1x_2U2x_2T8x_LB_um.pdf':
#
# 3. Tips
#   > When creating the library at the beginning of the design phase, the following command must be
#     used : "create_mw_lib –technology $TECH –mw_reference_library $REF"
#       * $TECH represent the path of the technology file
#       * $REF should first have listet the unitTile to be loaded, followed by the respective
#         standard cells PR directoryies.
#   > You should then choose which is your tile, depending on the standard cells used (8T or 12T)
#       * "set_fp_strategy -unit_tile_name <Cell name>" where <Cell name> may be unit8T or unit12T
#       
create_mw_lib -technology "$TECHNOLOGY_FILE" -mw_reference_library "$MW_REFERENCE_LIBRARIES" -open ${MW_LIBRARY_NAME}

# Floor plan strategy
set_fp_strategy -unit_tile_name unit12T
set_tlu_plus_files -max_tluplus "$tlup_max" -min_tluplus "$tlup_min" -tech2itf_map "$tlup_map"
check_tlu_plus_files

#
# Read gate-level netlist. After reading the netlist each cell will be placed on top of each other at (0,0).
# After all cells are loaded, they are ready to be reorganized (placed) in according to some 
# floorplan.
#
read_verilog "$VERILOG_NETLIST"
uniquify_fp_mw_cel
read_sdc "$SDC_CONSTRAINT"

#
# Generate a floorplan for the cells to be placed in. For more info read the manual. If desiered, 
# create the vdd! and gnd! power rings.
# icc_shell> man create_floorplan
# icc_shell> man create_rectilinear_rings
#
create_floorplan -control_type aspect_ratio -core_utilization 0.5

#
# Place the cells within the floorplan. If placement fails
# icc_shell> man place_opt
#
set_separate_process_options -placement false
place_opt -effort high

#
# Route all wires between the .
# icc_shell> man route_opt
#
set_separate_process_option -routing false
route_opt -effort high

#
# Save the results to the library
#
save_mw_cel -design "${DESIGN_NAME}.CEL;1"

