#
# Template TCL-script for sweeping shell script. Part of the Aardvark Design
# Sweep tool for Design Compiler sweeps.
#

# Maximum number of CPU cores allowed for parallel execution.
set_host_options -max_cores "__MAX_CORES__"

# Input files. Set by design sweep tool.
set HDL_FILES [list __HDL_FILES__]

# Period to run. Set by design sweep tool.
set PERIOD "__PERIOD__"

# Design clock name. Set by design sweep tool.
set CLOCK_NAME "__CLOCK_NAME__"

# Design generics. Set by design sweep tool.
set DESIGN_GENERICS "__DESIGN_GENERICS__"

# Top level design entity name. Set by design sweep tool.
set DESIGN_NAME "__DESIGN_NAME__"

# Retiming option. Set by design sweep tool.
set RETIME_ENABLE "__RETIME_ENABLE__"

# Export gate-level netlist from synth
set EXPORT_NETLIST "__EXPORT_NETLIST__"

# Read input files.
foreach hdl_file $HDL_FILES {
    set analyze_return_code [analyze -format vhdl "$hdl_file"]

    # Exit code 1 marks error of analyze.
    if { "$analyze_return_code" == 0 } { exit 1 }
}

# Elaborate design with generic. Design Compiler return code 2 is used for
# errornous elaboration stage.
set el_return_code [elaborate $DESIGN_NAME -parameters "$DESIGN_GENERICS"]
if { "$el_return_code" == 0 } { exit 2 }

# Create clock object. Design Compiler return code 3 is used for errornous
# create_clock stage.
set clk_return_code [create_clock "$CLOCK_NAME" -period "$PERIOD"]
if { "$clk_return_code" == 0 } { exit 3 }

# Set input and output delay constraints.
set_input_delay -max 0.0 -clock "$CLOCK_NAME" [remove_from_collection [all_inputs] [get_ports "$CLOCK_NAME"]]
set_output_delay -max 0.0 -clock "$CLOCK_NAME" [all_outputs]


# Compile design. Design Compiler return code 4 is used for errornous compile
# stage.
if { "$RETIME_ENABLE" == "YES" } {
    set_optimize_registers
    set compile_return_code [compile_ultra -retime]
    if { "$compile_return_code" == 0 } { exit 4 }
} else {
    set compile_return_code [compile_ultra]
    if { "$compile_return_code" == 0 } { exit 4 }
}

# Report synthesis timing, area and power results.
report_timing > "timing.txt"
report_area > "area.txt" 
report_power > "power.txt" 

# Export post-synthesis gate-level netlist if desiered
if { "$EXPORT_NETLIST" == "YES" } {
    # Standard Verilog gate-level netlist
    write -format verilog -hierarchy -output "${DESIGN_NAME}-post-synth.v"

    # Synopsys DDC gate-level netlist (combined netlist and design constraints)
    write -format ddc -hierarchy -output "${DESIGN_NAME}-post-synth.ddc"

    # Synthesis constraint information
    write_sdc -nosplit "${DESIGN_NAME}-post-synth.sdc"
}

# Exit Design Compiler.
exit 0
